/***
* Peanut.js
* Peanut plugin for Toon Boom Harmony
* Copyright NullBrush Lab Ltd. 2017
* Author: Aaron Shea
* Last Mod: 11/3/2017
***/

//// Settings Object
SETTINGS = {
	file: {
		outPath: ""
	}
}

//// Utils

/*
* Basic Logging Function
*/
function printMsg(msg)
{
	MessageLog.trace("[Peanut] " + msg);
	System.println("[Peanut] " + msg);
}

/*
* UI Loader Helper
*/
function loadUI(path, uiName)
{
	var uiPath = path + "/" + uiName;
	var ui = UiLoader.load(uiPath, path);

	return ui;
}

/*
* Kick off a render
*/
function PEANUT_render(startFrame, endFrame, width, height, dither, limit, colorLimit)
{
	printMsg("Going to create GIF from frame " + startFrame + " to " + endFrame + " at " + width + "x" + height);
	
	printMsg("Dithering: " + dither);

	if (limit)
		printMsg("Limit Quantize Colors: " + colorLimit);

	var ramUsageEst = width * height * 4;
	ramUsageEst *= (endFrame - startFrame);

	printMsg("Estimated bytes in memory: " + ramUsageEst);

	// Is this render going to take up more than ~2.5GB of RAM?
	// If so, make sure they REALLY want to do this
	if (ramUsageEst > 2621440000)
	{
		var confirm = MessageBox.warning("WARNING\n\nThe number of frames selected and resolution specified might use a lot of memory.\nIt is recommended to save your scene first!\nConfirm you want to continue by clicking 'Retry'");

		if (confirm != QMessageBox.Retry)
		{
			// They aborted
			return false;
		}
	}

	// Create a runtime
	var runtime = PeanutCreateRuntime();

	// Create the export handler
	var hld = new ExportHandler();
	hld.runtime = runtime;

	// Attach the handler
	render.setRenderDisplay(translator.tr("Unconnected_Display"));
	render.frameReady.connect(hld, hld.frameReady);
	render.renderFinished.connect(hld, hld.exportFinished);

	// Set the render width and height
	render.setResolution(width, height);

	// Tell the runtime to start a session
	runtime.start(scene.getFrameRate(), SETTINGS.file.outPath, width, height, dither, limit, colorLimit);

	// Tell TB to render the frames
	render.renderScene(startFrame, endFrame);

	return true;
}

//// Export Handler
function ExportHandler() {}

/*
* Push a frame into the peanut runtime
*/
ExportHandler.prototype.frameReady = function(frameIn, frameCel)
{
	// When TB finishes a frame we push it into the native plugin script interface
	PeanutSendFrame(frameIn, frameCel, this.runtime);
}

/*
* Tell runtime we're done, and to save the final gif
*/
ExportHandler.prototype.exportFinished = function()
{
	printMsg("Render done!");
	this.runtime.saveFile(); // Saves to the output path passed into start()
}

//// UI stuff starts here

/*
* Show the about window
*/
function PEANUT_AboutWindow()
{
	var scriptFolder = PeanutGetFolder() + "/resources";

	var ui = loadUI(scriptFolder, "about.ui");
	ui.show();
}

/*
* Show the export window
*/
function PEANUT_ExportWindow()
{
	var scriptFolder = PeanutGetFolder() + "/resources";

	var ui = loadUI(scriptFolder, "export.ui");

	var uiController = new MainUiCtrl(ui);

	uiController.exec();
}

//// Export UI Controller

function MainUiCtrl(loadedUi)
{
	this.ui = loadedUi;

	this.exec = function()
	{
		this.ui.finished.connect(this, saveSettings);
		this.ui.exec();
	}

	// Button Bindings

	this.close = function()
	{
		saveSettings();
		this.ui.close();
	}

	this.doRender = function()
	{
		// Close the export UI
		this.close();

		// Show the export status window
		PeanutShowStatusWindow();

		// Render time!
		var result = PEANUT_render(
			this.ui.frameProps.frameStartBox.value,
			this.ui.frameProps.frameEndBox.value,
			this.ui.sizeProps.widthBox.value,
			this.ui.sizeProps.heightBox.value,
			this.ui.advProps.ditherBox.checked,
			this.ui.advProps.colorLimitBox.checked,
			parseInt(this.ui.advProps.limitNumBox.currentText)
		);

		if (result)
		{
			MessageBox.information("Peanut saved GIF to " + SETTINGS.file.outPath);
		}
	}

	this.selectFile = function()
	{
		var path = QFileDialog.getSaveFileName(null, "Pick a place to save your GIF!", preferences.getString("PEANUT_OUTFILE_GLOBAL", scene.currentProjectPath()), "*.gif");
		if (path == "")
		{
			return;
		}
		this.ui.fileProps.filePathText.setText(path);
		this.ui.exportButton.setEnabled(true);
		SETTINGS.file.outPath = path;
	}

	// Setting reset methods
	this.loadSizeSettings = function()
	{
		// Reset options
		this.ui.sizeProps.widthBox.setValue(scene.currentResolutionX());
		this.ui.sizeProps.heightBox.setValue(scene.currentResolutionY());
		this.ui.sizeProps.percentBox.setValue(100);
	}

	this.loadFrameSettings = function()
	{
		// Reset options
		this.ui.frameProps.frameStartBox.setValue(1);
		this.ui.frameProps.frameEndBox.setValue(frame.numberOf());
	}

	this.ui.closeButton.pressed.connect(this, this.close);
	this.ui.exportButton.pressed.connect(this, this.doRender);
	this.ui.fileProps.pickPathButton.pressed.connect(this, this.selectFile);

	// Frame options
	this.ui.frameProps.frameStartBox['valueChanged(int)'].connect(this, function(newNum) {
		if (newNum <= 0)
		{
			this.ui.frameProps.frameStartBox.setValue(1);
		}
	});

	this.ui.frameProps.frameEndBox['valueChanged(int)'].connect(this, function(newNum) {
		if (newNum > frame.numberOf())
		{
			this.ui.frameProps.frameEndBox.setValue(frame.numberOf());
		}
	});

	this.ui.frameProps.toggled.connect(this, function(checked) {
		if (!checked)
		{
			this.loadFrameSettings();
		}
	});

	// Resolution options
	this.ui.sizeProps.widthBox['valueChanged(int)'].connect(this, function(newNum) {
		if (newNum > scene.currentResolutionX())
		{
			this.ui.sizeProps.widthBox.setValue(scene.currentResolutionX());
		}
	});

	this.ui.sizeProps.heightBox['valueChanged(int)'].connect(this, function(newNum) {
		if (newNum > scene.currentResolutionY())
		{
			this.ui.sizeProps.heightBox.setValue(scene.currentResolutionY());
		}
	});

	// Percent box
	this.ui.sizeProps.percentBox['valueChanged(int)'].connect(this, function(newNum) {
		var newWidth = scene.currentResolutionX() * (newNum / 100);
		var newHeight = scene.currentResolutionY() * (newNum / 100);

		this.ui.sizeProps.widthBox.setValue(newWidth);
		this.ui.sizeProps.heightBox.setValue(newHeight);
	});

	this.ui.sizeProps.toggled.connect(this, function(checked) {
		if (!checked)
		{
			this.loadSizeSettings();
		}
	});

	// Initial Settings GO!
	this.loadFrameSettings();
	this.loadSizeSettings();
}

//// Settings
function saveSettings(uiController)
{
	// Saves a nicer path so we can open it later when looking for a place to save
	if (SETTINGS.file.outPath != "")
	{
		var pathParts = SETTINGS.file.outPath.split('/');
		pathParts.pop();

		var globalOutPath = pathParts.join('/');

		preferences.setString("PEANUT_OUTFILE_GLOBAL", globalOutPath);
	}
}